const fs = require('fs');

function hslToRgb([h, s, l]){
  h = ((h % 360) + 360) % 360;

  let c = (1 - Math.abs(2 * l - 1)) * s, x = c * (1 - Math.abs((h / 60) % 2 - 1)), m = l - c / 2;

  return [
      [c, x, 0], //   0 <= h < 60
      [x, c, 0], //  60 <= h < 120
      [0, c, x], // 120 <= h < 180
      [0, x, c], // 180 <= h < 240
      [x, 0, c], // 240 <= h < 300
      [c, 0, x], // 300 <= h < 360
  ][(h / 60) | 0].map(v => v + m);
}

function luminance(rgb) {
  var a = rgb.map(function (v) {
      return v <= 0.03928
          ? v / 12.92
          : Math.pow( (v + 0.055) / 1.055, 2.4 );
  });
  return a[0] * 0.2126 + a[1] * 0.7152 + a[2] * 0.0722;
}

function hsrlToRgb(h, s, rl, round = Math.round) {
  let a = 1, b = 0, c = 0, mid = 0.5, midl;

  while (a - b > 0.0001) {
      c++;
      mid = (a + b) / 2, midl = luminance(hslToRgb([h, s, mid]));
      if (midl > rl) {
          a = mid;
      } else if (midl < rl) {
          b = mid;
      } else {
          break;
      }
  }

  return hslToRgb([h, s, mid]).map(v => round(v * 255));
}

function rgbToHex(rgb) {
  return '#' + rgb.map(v => v.toString(16).padStart(2, '0')).join('');
}

function contrast(a, b) {
  return (a + 0.05) / (b + 0.05);
}

function testContrast(label, rgb, test) {
  let a = luminance(rgb.map(v => v / 255)),
    b = luminance(test.map(v => v / 255)),
    c = Math.max(contrast(a, b), contrast(b, a));

  if (c < 4.5) {
    console.log('[' + label + ']', 'Contrast of', c, 'does not pass 4.5 threshold!', rgb, test);
  }

  return Math.round(c * 100) / 100;
}

let filename = 'accessible-colors.scss', contents = [
  '// ' + filename + ' - colors generated with accessibility-friendly color contrast',
  '',
  '// contrast ratio must be 4.5+ for AA accessibility',
  '// white and black have a contrast ratio of 21 together',
  '',
  '// light and dark colors have contrast ratio of 4.5+',
  '// extra light and dark colors have even higher contrast ratios',
  '// so hue and saturation is interchangable',
  '',
];

function generateColors(hue, sat) {
  let baselum = 0.1775, darklum = baselum * 0.7, color = hsrlToRgb(hue, sat, baselum),
    background = hsrlToRgb(hue, sat, darklum, Math.floor),
    foreground = hsrlToRgb(hue, sat, ((darklum + 0.05) * 4.5 - 0.05), Math.ceil),
    xl = hsrlToRgb(hue, sat, 0.85, Math.ceil);
  contents.push('// hue: ' + hue + ', saturation: ' + (sat * 100) + '%');
  contents.push('$color_' + hue + '_' + Math.round(sat * 100) +  ': ' + rgbToHex(color) + '; // Contrast on white & black: ' + testContrast('color on white ' + hue + ' ' + sat, color, [255, 255, 255]) + ' & ' +  + testContrast('color on black ' + hue + ' ' + sat, color, [0, 0, 0]));
  contents.push('$color_light_' + hue + '_' + Math.round(sat * 100) +  ': ' + rgbToHex(foreground) + '; $color_dark_' + hue + '_' + Math.round(sat * 100) +  ': ' + rgbToHex(background) + '; // Contrast against each other: ' + testContrast('colors on each other ' + hue + ' ' + sat, foreground, background));
  contents.push('$color_extra_light_' + hue + '_' + Math.round(sat * 100) +  ': ' + rgbToHex(xl) + '; // Contrast against dark color: ' + testContrast('xl on dark ' + hue + ' ' + sat, background, xl));
}

generateColors(0, 0);
contents.push('');

for (let hue = 0; hue < 360; hue += 15) {
  [1, 0.8, 0.6, 0.4, 0.2].forEach(sat => {
    generateColors(hue, sat);
  });
  contents.push('');
}  

contents = contents.join('\n');

fs.writeFileSync('scss/' + filename, contents);
